/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * This code developed by The Department of Biomedical and Health Informatics at 
 * at the University of Washington (http://www.bhi.washington.edu)
 *
 */
 
package edu.washington.bhi.protegex.storage.xml;

import java.io.*;
import org.w3c.dom.*;


/* Writes out XML from generic DOM node
 */
public class XMLPrinter extends XMLBase{
	
	//variables
	private final static int BUFFER_SIZE = 100000;
	
	/** Print writer. */
    protected PrintWriter printwriter;

	public XMLPrinter()
	{
	}
	
	private Writer getWriter(String filename) {
        Writer writer = null;
        if (filename != null) {
		    try{
				writer = new FileWriter(filename, false);
				writer = new BufferedWriter(writer, BUFFER_SIZE);
		    }
		    	catch(Exception e){
		    }
        }
        return writer;
    }
    
    
    //walk the DOM tree and write out as u go
    private void walk(Node node)
    {	
        int type = node.getNodeType();
        switch(type)
        {
            case Node.DOCUMENT_NODE:
            {
              printwriter.println("<?xml version=\"1.0\" encoding=\""+
                                "UTF-8" + "\"?>");  
              break;                  
            }//end of document
            case Node.ELEMENT_NODE:
            {
                printwriter.print('<' + node.getNodeName() );
                NamedNodeMap nnm = node.getAttributes();
                if(nnm != null )
                {
                    int len = nnm.getLength() ;
                    Attr attr;
                    for ( int i = 0; i < len; i++ )
                    {
                        attr = (Attr)nnm.item(i);
                        printwriter.print(' ' 
                             + attr.getNodeName()
                             + "=\""
                             + attr.getNodeValue()
                             +  '"' );
                    }
                }
                printwriter.print(">");
                
                break;
                
            }//end of element
            case Node.ENTITY_REFERENCE_NODE:
            {
               
               printwriter.print('&' + node.getNodeName() + ';' );
               break;
                
            }//end of entity
            case Node.CDATA_SECTION_NODE:
            {
                    printwriter.print( "<![CDATA[" 
                            + node.getNodeValue()
                            + "]]>" );
                     break;       
                
            }
            case Node.TEXT_NODE:
            {
                printwriter.print(node.getNodeValue());
                break;
            }
            case Node.PROCESSING_INSTRUCTION_NODE:
            {
                printwriter.print("<?" 
                    + node.getNodeName() ) ;
                String data = node.getNodeValue();
                if ( data != null && data.length() > 0 ) {
                    printwriter.print(' ');
                    printwriter.print(data);
                }
                printwriter.println("?>");
                break;

             }
        }//end of switch
                
        //recurse
        for(Node child = node.getFirstChild(); child != null; child = child.getNextSibling())
        {
            walk(child);
            printwriter.flush();
        }
        
       //without this the ending tags will miss
        if ( type == Node.ELEMENT_NODE )
        {
            printwriter.print("</" + node.getNodeName() + ">");
        }
        printwriter.flush();
    }//end of walk
        
    public void printXML(Node doc, String filename)
    {
		Writer writer = getWriter(filename);
		
		//initialize the printwriter  		
    	printwriter = new PrintWriter(writer);
        walk(doc);
    }
}